@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2017-2022, QIIME 2 development team.
+# Copyright (c) 2017-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -98,11 +98,14 @@ def _precision_recall_fscore(exp, obs, sample_weight=None):
 def _index_is_subset(series1, series2, name):
     ix1 = series1.index
     ix2 = series2.index
+
     if set(ix1) < set(ix2):
         raise ValueError(
             'Observed and expected ids do not match. Missing '
             'ids not found in {0} ids: {1}'.format(
-                name, set(ix1) - (set(ix2))))
+                name, set(ix2) - set(ix1)
+            )
+        )
 
 
 def _prf_to_dataframe(exp, obs, sample_weight=None, level_range=range(0, 7)):
