@@ -1,30 +1,34 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2017-2022, QIIME 2 development team.
+# Copyright (c) 2017-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
 # The full license is in the file LICENSE, distributed with this software.
 # ----------------------------------------------------------------------------
-
+import importlib
 import q2_quality_control
 from qiime2.plugin import (Str, Plugin, Choices, Range, Float, Int, Bool,
-                           MetadataColumn, Categorical, Citations, TypeMap,
-                           Visualization, TypeMatch)
+                           Threads, MetadataColumn, Visualization,
+                           Categorical, Citations, TypeMap,
+                           TypeMatch, Metadata, Collection)
 from q2_types.feature_data import FeatureData, Sequence, Taxonomy
 from q2_types.sample_data import SampleData
 from q2_types.per_sample_sequences import (
     SequencesWithQuality, PairedEndSequencesWithQuality)
-from q2_types.feature_table import FeatureTable, RelativeFrequency
+from q2_types.feature_table import FeatureTable, RelativeFrequency, Frequency
 from q2_types.bowtie2 import Bowtie2Index
-
 from .quality_control import (exclude_seqs, evaluate_composition,
                               evaluate_seqs, evaluate_taxonomy)
+from .decontam import (decontam_identify, decontam_remove)
+from ._pipelines import (decontam_identify_batches)
 from ._filter import bowtie2_build, filter_reads
-
+from ._threshold_graph import (decontam_score_viz)
+from ._stats import DecontamScore, DecontamScoreFormat, DecontamScoreDirFmt
 
 citations = Citations.load('citations.bib', package='q2_quality_control')
 
 plugin = Plugin(
+
     name='quality-control',
     version=q2_quality_control.__version__,
     website='https://github.com/qiime2/q2-quality-control',
@@ -36,7 +40,7 @@ plugin = Plugin(
         'Plugin for quality control of feature and sequence data.')
 )
 
-
+_DECON_METHOD_OPT = {'frequency', 'prevalence', 'combined'}
 seq_inputs = {'query_sequences': FeatureData[Sequence],
               'reference_sequences': FeatureData[Sequence]}
 
@@ -63,7 +67,7 @@ filter_input = {'demultiplexed_sequences': 'The sequences to be trimmed.',
 filter_output = {'filtered_sequences': 'The resulting filtered sequences.'}
 
 filter_parameters = {
-    'n_threads': Int % Range(1, None),
+    'n_threads': Threads,
     'mode': Str % Choices(['local', 'global']),
     'sensitivity': Str % Choices([
         'very-fast', 'fast', 'sensitive', 'very-sensitive']),
@@ -97,7 +101,7 @@ plugin.methods.register_function(
                 'perc_identity': Float % Range(0.0, 1.0, inclusive_end=True),
                 'evalue': Float,
                 'perc_query_aligned': Float,
-                'threads': Int % Range(1, None),
+                'threads': Threads,
                 'left_justify': P_left_justify,
                 },
     outputs=[('sequence_hits', FeatureData[Sequence]),
@@ -115,7 +119,7 @@ plugin.methods.register_function(
             'Percent of query sequence that must align to reference in order '
             'to be accepted as a hit.'),
         'threads': (
-            'Number of jobs to execute. Only applies to vsearch method.'),
+            'Number of threads to use. Only applies to vsearch method.'),
         'left_justify': ('Reject match if the pairwise alignment begins with '
                          'gaps'),
     },
@@ -291,13 +295,208 @@ plugin.methods.register_function(
 plugin.methods.register_function(
     function=bowtie2_build,
     inputs={'sequences': FeatureData[Sequence]},
-    parameters={'n_threads': Int % Range(1, None)},
+    parameters={'n_threads': Threads},
     outputs=[('database', Bowtie2Index)],
     input_descriptions={
         'sequences': 'Reference sequences used to build bowtie2 index.'},
-    parameter_descriptions={'n_threads': 'Number of threads to launch'},
+    parameter_descriptions={'n_threads': 'Number of threads to launch.'},
     output_descriptions={'database': 'Bowtie2 index.'},
     name='Build bowtie2 index from reference sequences.',
     description='Build bowtie2 index from reference sequences.',
     citations=[citations['langmead2012fast']]
 )
+
+# Decontam Actions
+plugin.methods.register_function(
+    function=decontam_identify,
+    inputs={'table': FeatureTable[Frequency]},
+    parameters={'metadata': Metadata,
+                'method': Str % Choices(_DECON_METHOD_OPT),
+                'freq_concentration_column': Str,
+                'prev_control_column': Str,
+                'prev_control_indicator': Str},
+    outputs=[('decontam_scores', FeatureData[DecontamScore])],
+    input_descriptions={
+        'table': ('Feature table which contaminate sequences '
+                  'will be identified from')
+    },
+    parameter_descriptions={
+        'metadata': ('metadata file indicating which samples in the '
+                     'experiment are control samples, '
+                     'assumes sample names in file correspond '
+                     'to the `table` input parameter'),
+        'method': ('Select how to which method '
+                   'to id contaminants with; '
+                   'Prevalence: Utilizes control ASVs/OTUs '
+                   'to identify contaminants, '
+                   'Frequency: Utilizes sample concentration '
+                   'information to identify contaminants, '
+                   'Combined: Utilizes both Prevalence and '
+                   'Frequency methods when identifying '
+                   'contaminants'),
+        'freq_concentration_column': ('Input column name that has '
+                                      'concentration information for '
+                                      'the samples'),
+        'prev_control_column': ('Input column name containing '
+                                'experimental or control '
+                                'sample metadata'),
+        'prev_control_indicator': ('indicate the '
+                                   'control sample identifier '
+                                   '(e.g. "control" or "blank")')
+    },
+    output_descriptions={
+        'decontam_scores': ('The resulting table of scores '
+                            'from the decontam algorithm '
+                            'which scores each feature on '
+                            'how likely they are to be a '
+                            'contaminant sequence')
+
+    },
+    name='Identify contaminants',
+    description=('This method identifies contaminant sequences from an '
+                 'OTU or ASV table and reports them to the user')
+)
+
+plugin.methods.register_function(
+    function=decontam_remove,
+    inputs={'decontam_scores': FeatureData[DecontamScore],
+            'table': FeatureTable[Frequency],
+            'rep_seqs': FeatureData[Sequence]},
+    parameters={'threshold': Float % Range(0.0, 1.0, inclusive_end=True)},
+    outputs=[('filtered_table', FeatureTable[Frequency]),
+             ('filtered_rep_seqs', FeatureData[Sequence])],
+    input_descriptions={
+        'decontam_scores': ('Pre-feature decontam scores.'),
+        'table': ('Feature table from which contaminants will be removed.'),
+        'rep_seqs': ('Feature representative sequences from which '
+                     'contaminants will be removed.')
+    },
+    parameter_descriptions={
+        'threshold': ('Decontam score threshold. Features with a score less '
+                      'than or equal to this threshold will be removed.')
+    },
+    output_descriptions={
+        'filtered_table': ('Feature table with contaminants removed.'),
+        'filtered_rep_seqs': ('Feature representative sequences with '
+                              'contaminants removed.')
+    },
+    name='Remove contaminants',
+    description=('Remove contaminant sequences from a feature table and '
+                 'the associated representative sequences.')
+)
+
+
+plugin.visualizers.register_function(
+    function=decontam_score_viz,
+    inputs={
+        'decontam_scores': Collection[FeatureData[DecontamScore]],
+        'table': Collection[FeatureTable[Frequency]],
+        'rep_seqs': FeatureData[Sequence]
+    },
+    parameters={
+        'threshold':  Float % Range(0.0, 1.0, inclusive_end=True),
+        'weighted': Bool,
+        'bin_size': Float % Range(0.0, 1.0, inclusive_end=True),
+    },
+    name='Generate a histogram representation of the scores',
+    description='Creates histogram based on the output of decontam identify',
+    input_descriptions={
+        'decontam_scores': 'Output from decontam identify '
+                           'to be visualized',
+        'table': 'Raw OTU/ASV table that was used '
+                 'as input to decontam-identify',
+        'rep_seqs': ('Representative Sequences table which contaminate '
+                     'sequences will be removed from')
+    },
+    parameter_descriptions={
+        'threshold': ('Select threshold cutoff for decontam algorithm scores'),
+        'weighted': ('weight the decontam scores by their '
+                     'associated read number'),
+        'bin_size': ('Select bin size for the histogram')
+    }
+)
+
+# Heirarchical Decontam format Pipeline
+plugin.pipelines.register_function(
+    function=decontam_identify_batches,
+    inputs={'table': FeatureTable[Frequency],
+            'rep_seqs': FeatureData[Sequence]},
+    parameters={'metadata': Metadata,
+                'split_column': Str,
+                'method': Str % Choices(_DECON_METHOD_OPT),
+                'filter_empty_features': Bool,
+                'freq_concentration_column': Str,
+                'prev_control_column': Str,
+                'prev_control_indicator': Str,
+                'threshold': Float,
+                'weighted': Bool,
+                'bin_size': Float
+                },
+    outputs=[('batch_subset_tables', Collection[FeatureTable[Frequency]]),
+             ('decontam_scores', Collection[FeatureData[DecontamScore]]),
+             ('score_histograms', Visualization)
+             ],
+    input_descriptions={
+        'table': ('Feature table which contaminate sequences '
+                  'will be identified from'),
+        'rep_seqs': ('Representative Sequences table which contaminate '
+                     'seqeunces will be removed from')
+    },
+    parameter_descriptions={
+        'metadata': ('metadata file indicating which samples in the '
+                     'experiment are control samples, '
+                     'assumes sample names in file correspond '
+                     'to the `table` input parameter'),
+        'split_column': (
+            'input metadata columns that you wish to subset the ASV table by'
+            'Note: Column names must be in quotes and delimited by a space'),
+        'method': ('Select how to which method '
+                   'to id contaminants with; '
+                   'Prevalence: Utilizes control ASVs/OTUs '
+                   'to identify contaminants, '
+                   'Frequency: Utilizes sample concentration '
+                   'information to identify contaminants, '
+                   'Combined: Utilizes both Prevalence and '
+                   'Frequency methods when identifying '
+                   'contaminants'),
+        'filter_empty_features': 'If true, features which are not present in '
+                                 'a split feature table are dropped.',
+        'freq_concentration_column': ('Input column name that has '
+                                      'concentration information for '
+                                      'the samples'),
+        'prev_control_column': ('Input column name containing '
+                                'experimental or control '
+                                'sample metadata'),
+        'prev_control_indicator': ('indicate the '
+                                   'control sample identifier '
+                                   '(e.g. "control" or "blank")'),
+        'threshold': ('Select threshold cutoff for decontam algorithm scores'),
+        'weighted': ('weight the decontam scores by their '
+                     'associated read number'),
+        'bin_size': ('Select bin size for the histogram')
+    },
+    output_descriptions={
+        'batch_subset_tables': (
+            'Directory where feature tables split based on metadata '
+            'and parameter split_column values should be written.'),
+        'decontam_scores': ('The resulting table of scores '
+                            'from the decontam algorithm '
+                            'which scores each feature on '
+                            'how likely they are to be a '
+                            'contaminant sequence'),
+        'score_histograms': (
+            'The vizulaizer histograms for all decontam score objects '
+            'generated from the pipeline')
+    },
+    name='Identify contaminants in Batch Mode',
+    description=(
+        'This method breaks an ASV table into '
+        'batches based on the given metadata and '
+        'identifies contaminant sequences from an '
+        'OTU or ASV table and reports them to the user')
+)
+plugin.register_formats(DecontamScoreFormat, DecontamScoreDirFmt)
+plugin.register_semantic_types(DecontamScore)
+plugin.register_semantic_type_to_format(
+    FeatureData[DecontamScore], DecontamScoreDirFmt)
+importlib.import_module('q2_quality_control._transformer')
