@@ -1,16 +1,15 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2017-2022, QIIME 2 development team.
+# Copyright (c) 2017-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
 # The full license is in the file LICENSE, distributed with this software.
 # ----------------------------------------------------------------------------
 
-import qiime2
 import biom
 from q2_types.feature_data import DNAFASTAFormat
 import pandas as pd
-
+import qiime2.util
 from ._blast import _search_seqs
 from ._utilities import (
     _evaluate_composition, _visualize, _pointplot_multiple_y)
@@ -23,7 +22,7 @@ left_justify_supported_methods = {'vsearch'}
 def exclude_seqs(query_sequences: DNAFASTAFormat,
                  reference_sequences: DNAFASTAFormat, method: str = 'blast',
                  perc_identity: float = 0.97, evalue: float = None,
-                 perc_query_aligned: float = 0.97, threads: str = 1,
+                 perc_query_aligned: float = 0.97, threads: int = 1,
                  left_justify: bool = False,
                  ) -> (pd.Series, pd.Series):
 
