@@ -1,11 +1,10 @@
 # ----------------------------------------------------------------------------
-# Copyright (c) 2017-2022, QIIME 2 development team.
+# Copyright (c) 2017-2023, QIIME 2 development team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
 # The full license is in the file LICENSE, distributed with this software.
 # ----------------------------------------------------------------------------
-
 import numpy as np
 import numpy.testing as npt
 import pandas as pd
@@ -15,7 +14,6 @@ from warnings import filterwarnings
 from qiime2.plugin.testing import TestPluginBase
 from q2_types.feature_data import DNAFASTAFormat
 import pandas.testing as pdt
-
 from q2_quality_control.quality_control import (
     exclude_seqs, evaluate_composition, evaluate_seqs, evaluate_taxonomy)
 from q2_quality_control._utilities import (
@@ -29,7 +27,6 @@ from q2_quality_control._evaluate_taxonomy import (
     _evaluate_taxonomy, _extract_taxa_names, _index_is_subset,
     _validate_indices_and_set_joining_mode)
 
-
 filterwarnings("ignore", category=UserWarning)
 
 
@@ -490,7 +487,7 @@ class EvaluateCompositionTests(QualityControlTestsBase):
     def test_evaluate_composition_dont_test_all_levels(self):
         empty_expectations = pd.DataFrame(
             columns=['s1', 's2', 's3']).astype(float)
-        empty_expectations.index.name = 'Taxon'
+        empty_expectations.set_index(pd.Index([], name='Taxon'), inplace=True)
         mc = self.misclassified.loc[[
             'k__Ag;p__Bg;c__Cg;o__Dg;f__Eg;g__Fg;s__Gg']]
         mc.index = ['k__Ag;p__Bg;c__Cg;o__Dg;f__Eg']
